;;; Warning!!! This file is derived from those in actions/menus/... do not edit here
;;AllowLineBreak
(StandAloneSelfEditDirective (cons "AllowLineBreak" "\\bar \"\"") #f "AllowLineBreak")
(d-DirectivePut-standalone-graphic "AllowLineBreak" "\n|\nDenemo\n24")
